
clear

% get list of all triliterals not of form XYY
load ../buckwalter_data/buckwalter buckwalter_roots_nongem;  
wordmat=buckwalter_roots_nongem;
% wordmat - list of triplets of consonants representing words.  Each consonant 
% is indicated by its number between 1 and 28

numwords=size(wordmat,1); % number of words

wordinput=getwords(wordmat); % convert to localist binary format

numnodes=size(wordinput,2); % number of nodes for input and output

% initialize weights
b=zeros(numnodes,1);
W=zeros(numnodes);

% noise level - controls variability of the answers network gives
eta=0.325;

% number of training epochs
numepochs=1e5;
bmat=zeros(numnodes,numepochs/100);  % stores b over training
Wmat=zeros(numnodes,numepochs/100);  % stores W over training

% learning parameter delta changes with time
deltainitial=.1;
deltafinal=.0001;
deltaratio=(deltafinal/deltainitial)^(1/numepochs);
deltavect=deltainitial*deltaratio.^(1:numepochs);

% this vector keeps track of the number of segments that the network gets wrong
% per word in every epoch of training
segswrong=zeros(numepochs,1);

% learning step
for j=1:numepochs

  if mod(j,10000)==0 % occasionally print out epoch number
     j
  end

  % choose learning rate for this step
  delta=deltavect(j);
 
  % pick random word for input and target
  target=wordinput(ceil(numwords*rand),:)';

  % apply net with noise
  output=W*target+b+eta*randn(numnodes,1);

  % choose word based on output
  roundoutput=round_to_word(output')';
  errb=roundoutput-target;
  errW=output-target;
      
  % train
  b=b-delta*errb;
  W=W-delta*errW*target';
  alpha=.001;
  W=W*(1-delta*alpha); % diminish weights by a bit

  % segments wrong
  segswrong(j)=sum(target>roundoutput); 
  % record b and diag(W) over time
  if mod(j,100)==0
    bmat(:,j/100)=b;
    Wmat(:,j/100)=diag(W);
  end
end

save autoparams W b eta

